@@ -123,7 +123,6 @@ def test_pixel_to_world_values_units_2d(gwcs_2d_shift_scale_quantity, x, y):
     # Check that they are the same (and implicitly in the same units)
     assert_allclose(u.Quantity(call_world).value, api_world)
 
-
     new_call_pixel = wcsobj.invert(*call_world, with_units=False)
     [assert_allclose(n, p) for n, p in zip(new_call_pixel, call_pixel)]
 
@@ -148,7 +147,6 @@ def test_pixel_to_world_values_units_1d(gwcs_1d_freq_quantity, x):
     # Check that they are the same (and implicitly in the same units)
     assert_allclose(u.Quantity(call_world).value, api_world)
 
-
     new_call_pixel = wcsobj.invert(call_world, with_units=False)
     assert_allclose(new_call_pixel, call_pixel)
 
@@ -207,6 +205,7 @@ def test_world_axis_object_classes_4d(gwcs_4d_identity_units):
                            'format': 'isot', 'scale': 'utc', 'precision': 3,
                            'in_subfmt': '*', 'out_subfmt': '*', 'location': None}
 
+
 def _compare_frame_output(wc1, wc2):
     if isinstance(wc1, coord.SkyCoord):
         assert isinstance(wc1.frame, type(wc2.frame))
@@ -234,7 +233,7 @@ def test_high_level_wrapper(wcsobj, request):
 
     # Remove the bounding box because the type test is a little broken with the
     # bounding box.
-    del wcsobj._pipeline[0][1].bounding_box
+    del wcsobj._pipeline[0].transform.bounding_box
 
     hlvl = HighLevelWCSWrapper(wcsobj)
 
@@ -267,7 +266,6 @@ def test_stokes_wrapper(gwcs_stokes_lookup):
 
     assert list(out) == ['I', 'Q', 'U', 'V']
 
-
     pixel_input = [[0, 1, 2, 3],
                    [0, 1, 2, 3],
                    [0, 1, 2, 3],
@@ -296,7 +294,6 @@ def test_stokes_wrapper(gwcs_stokes_lookup):
     assert np.isnan(np.array(out[0], dtype=float)).all()
     assert (out[1] == np.array(['Q', 'U'], dtype=object)).all()
 
-
     out = hlvl.pixel_to_world(1*u.pix)
 
     assert out == 'Q'
